@@ -1,10 +1,12 @@
 /*
- * manager.c - GNOME Volume Manager
+ * src/manager.c - GNOME Volume Manager
  *
  * Robert Love <rml@ximian.com>
  *
  * Licensed under the GNU GPL v2.  See COPYING.
  *
+ * (C) Copyright 2004 Novell, Inc.
+ *
  * gnome-volume-manager is a simple policy agent that sits on top of the stack
  * of system-level components including the kernel, hotplug, udev, and HAL.
  * Responding to HAL events, gnome-volume-manager implements automount,
@@ -13,10 +15,7 @@
 
 #include "config.h"
 
-#include <unistd.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
-#include <linux/cdrom.h>
+#include <string.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
@@ -53,64 +52,6 @@
 
 struct gvm_configuration config;
 
-/*
- * clipboard_get_func - dummy get_func for gtk_clipboard_set_with_data ()
- */
-static void
-clipboard_get_func (GtkClipboard *clipboard, GtkSelectionData *selection_data,
-		    guint info, gpointer user_data_or_owner)
-{
-	(void) clipboard;
-	(void) selection_data;
-	(void) info;
-	(void) user_data_or_owner;
-}
-
-/*
- * clipboard_clear_func - dummy clear_func for gtk_clipboard_set_with_data ()
- */
-static void
-clipboard_clear_func (GtkClipboard *clipboard, gpointer user_data_or_owner)
-{
-	(void) clipboard;
-	(void) user_data_or_owner;
-}
-
-/*
- * gvm_get_clipboard - try and get the CLIPBOARD_NAME clipboard
- *
- * Returns TRUE if successfully retrieved and FALSE otherwise.
- */
-static gboolean
-gvm_get_clipboard (void)
-{
-	static const GtkTargetEntry targets[] = { {CLIPBOARD_NAME, 0, 0} };
-	gboolean retval = FALSE;
-	GtkClipboard *clipboard;
-	Atom atom;
-
-	atom = gdk_x11_get_xatom_by_name (CLIPBOARD_NAME);
-
-	XGrabServer (GDK_DISPLAY ());
-
-	if (XGetSelectionOwner (GDK_DISPLAY (), atom) != None)
-		goto out;
-
-	clipboard = gtk_clipboard_get (gdk_atom_intern (CLIPBOARD_NAME, FALSE));
-
-	if (gtk_clipboard_set_with_data (clipboard, targets,
-					 G_N_ELEMENTS (targets),
-					 clipboard_get_func,
-					 clipboard_clear_func, NULL))
-		retval = TRUE;
-
-out:
-	XUngrabServer (GDK_DISPLAY ());
-	gdk_flush ();
-
-	return retval;
-}
-
 /*
  * gvm_load_config - synchronize gconf => config structure
  */
@@ -465,7 +406,8 @@ gvm_run_cdplayer (const char *device, const char *mount_point)
 /*
  * gvm_ask_mixed - if a mixed mode CD (CD Plus) is inserted, we can either
  * mount the data tracks or play the audio tracks.  How we handle that depends
- * on the user's configuration.
+ * on the user's configuration.  If the configuration allows either option,
+ * we ask.
  */
 static void
 gvm_ask_mixed (const char *udi)
@@ -525,41 +467,38 @@ gvm_run_cdburner (const char *device, const char *mount_point)
 				 mount_point);
 }
 
-#define BURNER (CDC_CD_R | CDC_CD_RW | CDC_DVD_R | CDC_DVD_RAM)
-
 /*
  * gvm_device_is_writer - is this device capable of writing CDs?
  */
 static gboolean
-gvm_device_is_writer (const char *device)
+gvm_device_is_writer (const char *udi)
 {
-	int type;
-	int fd;
+	if (hal_device_get_property_int (udi, "storage.cdr"))
+		return 1;
+	else if (hal_device_get_property_int (udi, "storage.cdrw"))
+		return 1;
+	else if (hal_device_get_property_int (udi, "storage.dvdr"))
+		return 1;
+	else if (hal_device_get_property_int (udi, "storage.dvdram"))
+		return 1;
 
-	fd = open (device, O_RDONLY | O_NONBLOCK | O_EXCL);
-	if (fd < 0) {
-		warn ("unable to open %s\n", device);
-		return FALSE;
-	}
-
-	type = ioctl (fd, CDROM_GET_CAPABILITY, CDSL_CURRENT);
-	close (fd);
-
-	if (type < 0)
-		return FALSE;
-	else
-		return ((type & BURNER) != 0);
+	return 0;
 }
 
 /*
  * gvm_cdrom_policy - There has been a media change event on the CD-ROM
- * associated with the UDI.  Enforce policy.
+ * associated with the given UDI.  Enforce policy.
  */
 static void
 gvm_cdrom_policy (const char *udi)
 {
-	char *device = NULL, *mount_point = NULL;
-	int type, fd;
+	char *type, *device = NULL;
+
+	type = hal_device_get_property_string (udi, "storage.cdrom.media_type");
+	if (!type) {
+		warn ("unable to get storage.cdrom.media_type for %s\n", udi);
+		goto out;
+	}
 
 	device = hal_device_get_property_string (udi, "block.device");
 	if (!device) {
@@ -567,92 +506,21 @@ gvm_cdrom_policy (const char *udi)
 		goto out;
 	}
 
-	fd = open (device, O_RDONLY | O_NONBLOCK | O_EXCL);
-	if (fd < 0) {
-		warn ("unable to open %s\n", device);
-		goto out;
-	}
-
-	type = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
-	close (fd);
-
-	switch (type) {
-	case CDS_AUDIO:		/* audio CD */
+	if (!strcmp (type, "audio")) {
 		gvm_run_cdplayer (device, device);
-		break;
-	case CDS_MIXED:		/* mixed mode CD (CD Plus) */
+	} else if (!strcmp (type, "mixed")) {
 		gvm_ask_mixed (udi);
-		break;
-	case CDS_DATA_1:	/* data CD */
-	case CDS_DATA_2:
-	case CDS_XA_2_1:
-	case CDS_XA_2_2:
+	} else if (!strcmp (type, "iso9660")) {
 		if (config.automount_media)
 			gvm_device_mount (device);
-		break;
-	case CDS_NO_INFO:	/* blank or invalid CD */
+	} else if (!strcmp (type, "blank")) {
 		if (gvm_device_is_writer (device))
 			gvm_run_cdburner (device, device);
-		break;
-	default:		/* should never see this */
-		break;
 	}
 
 out:
 	hal_free_string (device);
-	hal_free_string (mount_point);
-}
-
-/*
- * gvm_cdrom_has_media - ensure that the CD-ROM associated with the given UDI
- * indeed has media.
- *
- * Returns TRUE if there is media in the given device and FALSE otherwise.
- */
-static gboolean
-gvm_cdrom_has_media (const char *udi)
-{
-	gboolean has_media = FALSE;
-	char *device;
-	int fd;
-
-	device = hal_device_get_property_string (udi, "block.device");
-	if (!device) {
-		warn ("cannot get block.device\n");
-		return has_media;
-	}
-
-	fd = open (device, O_RDONLY | O_NONBLOCK | O_EXCL);
-	if (fd < 0) {
-		warn ("cannot open %s\n", device);
-		goto out;
-	}
-
-	if (ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT) == CDS_DISC_OK) {
-		int disctype;
-
-		disctype = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
-		switch (disctype) {
-		case CDS_NO_INFO:
-		case CDS_AUDIO:
-		case CDS_DATA_1:
-		case CDS_DATA_2:
-		case CDS_XA_2_1:
-		case CDS_XA_2_2:
-		case CDS_MIXED:
-			has_media = TRUE;
-			break;
-		default:
-			/* mo media detected */
-			break;
-		}
-	}
-
-	close (fd);
-out:
-	hal_free_string (device);
-
-	return has_media;
+	hal_free_string (type);
 }
 
 /*
@@ -678,8 +546,7 @@ gvm_media_changed (const char *udi)
 	}
 
 	if (!g_strcasecmp (media_type, "cdrom"))
-		if (gvm_cdrom_has_media (udi) == TRUE)
-			gvm_cdrom_policy (udi);
+		gvm_cdrom_policy (udi);
 
 	/* other media_types go here */
 
@@ -797,11 +664,11 @@ handle_device_removed (DBusMessage *message)
 static DBusHandlerResult
 handle_property_modify (DBusMessage *message)
 {
-	dbus_int32_t nr;
-	char *key;
 	dbus_bool_t added, removed;
 	DBusMessageIter iter;
+	dbus_int32_t nr;
 	const char *udi;
+	char *key;
 	int i;
 
 	udi = dbus_message_get_path (message);
